#
#
# Copyright 2020-21 British Broadcasting Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

__all__ = ["HAS_AGENSGRAPH", "HAS_NEO4J"]


try:
    import agensgraph  # noqa: F401
except ImportError:
    HAS_AGENSGRAPH = False
else:
    HAS_AGENSGRAPH = True


try:
    import neo4j  # noqa: F401
except ImportError:
    HAS_NEO4J = False
else:
    HAS_NEO4J = True
